/*	$Csoft: install-cgi.c,v 1.1 2004/04/28 06:04:34 vedge Exp $	*/

/*
 * Copyright (c) 2004 CubeSoft Communications, Inc.
 * <http://www.csoft.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <config/prefix.h>
#include <config/sysconfdir.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>

int verbose = 0;
int overwrite = 0;
char *app = NULL;

static void
printusage(void)
{
	printf("Usage: install-cgi [-vo] [script-name]\n");
}

static int
copy_file(const char *from, const char *to)
{
	char buf[1024];
	int sfd, lfd;
	ssize_t nr;

	if ((sfd = open(from, O_RDONLY)) == -1) {
		fprintf(stderr, "%s: %s\n", from, strerror(errno));
		return (-1);
	}
	if ((lfd = open(to, O_WRONLY|O_CREAT, 0700)) == -1) {
		fprintf(stderr, "%s: %s\n", to, strerror(errno));
		close(sfd);
		return (-1);
	}
	if (verbose) {
		fprintf(stderr, "%s -> %s", from, to);
	}
	while ((nr = read(sfd, buf, sizeof(buf))) != -1 && nr != 0) {
		if (write(lfd, buf, nr) < nr) {
			fprintf(stderr, "write error\n");
			goto fail;
		}
	}
	if (verbose) {
		fputc('\n', stderr);
	}
	close(lfd);
	close(sfd);
	return (0);
fail:
	close(lfd);
	close(sfd);
	return (-1);
}

int
main(int argc, char *argv[])
{
	char path[FILENAME_MAX];
	char cmd[1024];
	extern int optind;
	int c;

	while ((c = getopt(argc, argv, "vo?")) != -1) {
		switch (c) {
		case 'v':
			verbose = 1;
			break;
		case 'o':
			overwrite = 1;
			break;
		default:
			printusage();
			return (1);
		}
	}
	argc -= optind;
	argv += optind;

	if (argc < 1) {
		printusage();
		return (1);
	}
	app = argv[0];

	strlcpy(path, PREFIX, sizeof(path));
	strlcat(path, "/bin/", sizeof(path));
	strlcat(path, app, sizeof(path));
	if (copy_file(path, app) == -1)
		return (1);

	snprintf(cmd, sizeof(cmd), "cp -R %s/%s %s.conf", SYSCONFDIR, app,
	    app);
	system(cmd);
	return (0);
}
