/*	$Csoft: cgi.h,v 1.8 2005/07/06 07:36:37 vedge Exp $	*/
/*	Public domain	*/

#ifndef _LIBCGI_CGI_H_
#define _LIBCGI_CGI_H_

#include <config/have_bounded_attribute.h>
#include <config/have_format_attribute.h>
#include <config/have_nonnull_attribute.h>
#include <config/enable_nls.h>
#include <config/have_fastcgi.h>

#ifdef HAVE_BOUNDED_ATTRIBUTE
#define BOUNDED_ATTRIBUTE(t, a, b) __attribute__((__bounded__ (t,a,b)))
#else
#define BOUNDED_ATTRIBUTE(t, a, b)
#endif
#ifdef HAVE_FORMAT_ATTRIBUTE
#define FORMAT_ATTRIBUTE(t, a, b) __attribute__((__format__ (t,a,b)))
#else
#define FORMAT_ATTRIBUTE(t, a, b)
#endif
#ifdef HAVE_NONNULL_ATTRIBUTE
#define NONNULL_ATTRIBUTE(a) __attribute__((__nonnull__ (a)))
#else
#define NONNULL_ATTRIBUTE(a)
#endif

#ifdef HAVE_FASTCGI
#include <fcgi_stdio.h>
#else
#include <sys/types.h>
#include <stdio.h>
#endif

#include <sys/param.h>
#include <sysexits.h>

#define CGI_MAX_ARGS		256	/* Maximum urlencoded arguments */
#define CGI_ARG_KEY_MAX		256	/* Maximum length for argument keys */
#define CGI_LANGS_MAX		378	/* Maximum Accept-Language entries
					   (ISO 639 has 378 languages codes) */
#define CGI_CSETS_MAX		245	/* Maximum Accept-Charset entries
					   (IANA assigns 245 official names) */
#define CGI_URL_MAX		4096

#include <compat/strlcpy.h>
#include <compat/strlcat.h>
#include <compat/snprintf.h>
#include <compat/queue.h>

#define Free(p)			cgi_free((p))
#define Malloc(len)		cgi_malloc((len))
#define Realloc(p, len)		cgi_realloc((p), (len))
#define Strdup(s)		cgi_strdup((s))

#ifdef ENABLE_NLS
# include <libintl/libintl.h>
# define _(String) gettext(String)
# define gettext_noop(String) (String)
# define N_(String) gettext_noop(String)
#else
# undef _
# undef N_
# undef textdomain
# undef bindtextdomain
# define _(s) (s)
# define N_(s) (s)
# define textdomain(d)
# define bindtextdomain(p, d)
#endif

#if !defined(__BEGIN_DECLS) || !defined(__END_DECLS)
# if defined(__cplusplus)
#  define __BEGIN_DECLS	extern "C" {
#  define __END_DECLS	}
# else
#  define __BEGIN_DECLS
#  define __END_DECLS
# endif
#endif

#include <libcgi/var.h>

struct cgi_query;

/* Content-specific filter */
struct cgi_filter {
	enum cgi_filter_type {
		CGI_INPUT_FILTER,	/* Filter data read by the script */
		CGI_OUTPUT_FILTER	/* Filter data written to stdout */
	} type;
	char	*content_type;		/* Apply to data of this Content-type */
	size_t	(*func)(struct cgi_query *, void **, size_t); 	    /* Filter */
	TAILQ_ENTRY(cgi_filter) filters;
};

/* Argument to script (key=value pair). */
struct cgi_argument {
	enum cgi_argument_type {
		CGI_GET_ARGUMENT,		/* HTTP GET method */
		CGI_POST_ARGUMENT		/* HTTP POST method */
	} type;
	char	 key[CGI_ARG_KEY_MAX];		/* Key */
	char	*value;				/* Copy of value */
	TAILQ_ENTRY(cgi_argument) args;
};

#define COOKIE_MAX		20
#define COOKIE_NAME_MAX		48
#define COOKIE_VALUE_MAX	3807
#define COOKIE_EXPIRE_MAX	64
#define COOKIE_DOMAIN_MAX	48
#define COOKIE_PATH_MAX		128

/* HTTP cookie to return to the client */
struct cgi_cookie {
	char name[COOKIE_NAME_MAX];		/* Name of cookie */
	char value[COOKIE_VALUE_MAX];		/* Value of cookie */
	char expire[COOKIE_EXPIRE_MAX];		/* Expiration date or \0 */
	char domain[COOKIE_DOMAIN_MAX];		/* Domain match or \0 */
	char path[COOKIE_PATH_MAX];		/* Path attribute or \0 */
	int secure;				/* Require HTTPS */
	TAILQ_ENTRY(cgi_cookie) cookies;
};

/* Query from web server */
struct cgi_query {
	char	*script_name;			/* Value of SCRIPT_NAME */
	char	 script_url[CGI_URL_MAX];	/* Script URL + sessid */
	const char *res_type;			/* Response's Content-Type */
	char	*accept_langs[CGI_LANGS_MAX];	/* Accepted languages */
	char	*accept_csets[CGI_CSETS_MAX];	/* Accepted character sets */
	char	 lang[4];			/* Negotiated language */
	char	 cset[16];			/* Negotiated character set */
	TAILQ_HEAD(,cgi_argument) args;		/* GET/POST arguments */
	TAILQ_HEAD(,cgi_cookie) cookies;	/* HTTP cookies */
};

/* Set of keys */
struct cgi_keyset {
	char	**ents;
	int	 nents;
};

/* Script globals */
struct cgi {
	char	*name;				/* Description of the script */
	char	*copyright;			/* Copyright notice */
	char	*avail_langs[CGI_LANGS_MAX];	/* Available languages */
	int	 flags;
#define CGI_HTML_ERRORS	0x01		/* Send HTML errors to stdout */
#define CGI_PERSISTENT	0x02		/* Only run under FastCGI */
	void	(*destroy_fn)(void);

	char	*avail_csets[3];		/* Available character sets */

	TAILQ_HEAD(,cgi_filter) filters;	/* Content filter chain */
	SLIST_HEAD(,var)	vars;		/* User variables */
};

enum cgi_loglvl {
	LOG_EMERG,
	LOG_ALERT,
	LOG_CRIT,
	LOG_ERR,
	LOG_WARNING,
	LOG_NOTICE,
	LOG_INFO,
	LOG_DEBUG
};

extern struct cgi *cgi;

__BEGIN_DECLS
void	 cgi_init(struct cgi *);
void	 cgi_destroy(int, const char *, ...)
	     FORMAT_ATTRIBUTE(__printf__, 2, 3);
void	 cgi_free(void *);
void	*cgi_malloc(size_t);
void	*cgi_realloc(void *, size_t);
char	*cgi_strdup(const char *);

void	 error_set(const char *, ...)
	     FORMAT_ATTRIBUTE(__printf__, 1, 2)
	     NONNULL_ATTRIBUTE(1);
char	*error_get(void);

struct cgi_query *cgi_accept(void);
void		  cgi_begin(struct cgi_query *, const char *);
const char	 *cgi_get_arg(struct cgi_query *, const char *);
const char	 *cgi_lget_arg(struct cgi_query *, const char *, size_t);
void	 	  cgi_end(struct cgi_query *);
void		  cgi_add_filter(struct cgi_filter *);
char		 *cgi_url_escape(struct cgi_query *, const char *);
char		 *cgi_url_unescape(struct cgi_query *, const char *);

ssize_t	 cgi_write(struct cgi_query *, void *, size_t);
void	 cgi_printf(struct cgi_query *, const char *, ...)
	     FORMAT_ATTRIBUTE(__printf__, 2, 3)
	     NONNULL_ATTRIBUTE(2);
void	 cgi_log(enum cgi_loglvl, const char *, ...)
	     FORMAT_ATTRIBUTE(__printf__, 2, 3)
	     NONNULL_ATTRIBUTE(2);
int	 cgi_find_datafile(const char *, char *, size_t)
	     BOUNDED_ATTRIBUTE(__string__, 2, 3);
int	 cgi_map_datafile(const char *, char **, size_t *);
void	 cgi_get_time(void);

struct cgi_keyset *cgi_get_keyset(struct cgi_query *, const char *);
void		   cgi_free_keyset(struct cgi_keyset *);

__inline__ char	  *cgi_get_cookie(struct cgi_query *, const char *);
struct cgi_cookie *cgi_set_cookie(struct cgi_query *, const char *,
		                  const char *, ...);

int	 sort_fields(const void *, const void *, const char *, int);
__END_DECLS

#endif /* _LIBCGI_CGI_H_ */
