/*	$Csoft: html.c,v 1.5 2004/01/03 04:10:19 vedge Exp $	*/

/*
 * Copyright (c) 2003, 2004 CubeSoft Communications, Inc.
 * <http://www.csoft.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <libcgi/cgi.h>

#include <sys/param.h>

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

#include "html.h"

/*
 * Write an HTML document to the standard output, appling the chain of
 * input filters which are registered for the `text/html' content-type.
 */
int
html_output(struct cgi_query *q, const char *name)
{
	char file[FILENAME_MAX];
	char path[MAXPATHLEN];
	struct cgi_filter *fil;
	void *buf;
	size_t len;
	ssize_t rv;
	int fd, e;

	cgi_get_time();

	strlcpy(file, name, sizeof(file));
	if ((e = strlcat(file, ".html.", sizeof(file))) >= sizeof(file)) {
		error_set(_("Document name is too big."));
		goto fail_open;
	}
	strlcpy(&file[e], q->lang, sizeof(file)-e);
	strlcat(&file[e], ".", sizeof(file)-e);
	strlcat(&file[e], q->cset, sizeof(file)-e);
	if ((cgi_find_datafile(file, path, sizeof(path))) == -1) {
		strlcpy(&file[e], cgi->avail_langs[0], sizeof(file)-e);
		strlcat(&file[e], ".", sizeof(file)-e);
		strlcat(&file[e], q->cset, sizeof(file)-e);
		if ((cgi_find_datafile(file, path, sizeof(path))) == -1) {
			strlcpy(&file[e], cgi->avail_langs[0], sizeof(file)-e);
			if ((cgi_find_datafile(file, path, sizeof(path)))
			    == -1)
				goto fail_open;
		}
	}

	if ((fd = open(path, O_RDONLY)) == -1) {
		error_set("%s: %s", path, strerror(errno));
		goto fail_open;
	}
	if ((len = lseek(fd, 0, SEEK_END)) <= 0 ||
	    lseek(fd, 0, SEEK_SET) == -1) {
	    	error_set(_("Data file `%s' is empty"), name);
		goto fail_open;
	}

	if ((buf = malloc(len)) == NULL) {
		error_set(_("Out of memory for file `%s'"), name);
		goto fail_mem;
	}
	if ((rv = read(fd, buf, len)) == -1) {
		error_set("%s: %s", name, strerror(errno));
		goto fail_read;
	}
	TAILQ_FOREACH(fil, &cgi->filters, filters) {
		if (fil->type == CGI_INPUT_FILTER &&
		    strcmp(fil->content_type, "text/html") == 0)
			len = fil->func(q, &buf, len);
	}
	cgi_write(q, buf, len);
	close(fd);
	return (0);
fail_read:
	free(buf);
fail_mem:
	close(fd);
fail_open:
	cgi_destroy(EX_NOINPUT, "%s", error_get());
	return (-1);
}

/* Write a generic HTML error document to the standard output. */
void
html_error(struct cgi_query *q, const char *fmt, ...)
{
	char errmsg[BUFSIZ];
	va_list ap;

	va_start(ap, fmt);
	vsnprintf(errmsg, sizeof(errmsg), fmt, ap);
	va_end(ap);

	var_set("error", errmsg);
	html_output(q, "error");
}

void
html_error_set(const char *fmt, ...)
{
	char errmsg[BUFSIZ];
	struct var *var;
	va_list ap;

	va_start(ap, fmt);
	vsnprintf(errmsg, sizeof(errmsg), fmt, ap);
	va_end(ap);

	var = var_set("error", _("<h2 class='error'>Error</h2>"));
	var_append(var, "<ul><li class='error'>%s</li></ul>", errmsg);
}

void
html_success_set(const char *fmt, ...)
{
	char succmsg[BUFSIZ];
	struct var *var;
	va_list ap;

	va_start(ap, fmt);
	vsnprintf(succmsg, sizeof(succmsg), fmt, ap);
	va_end(ap);

	var = var_set("error", _("<h2 class='success'>Success</h2>"));
	var_append(var, "<ul><li class='success'>%s</li></ul>", succmsg);
}

