/*	$Csoft: mime.c,v 1.4 2005/07/06 07:36:38 vedge Exp $	*/

/*
 * Copyright (c) 2003, 2004 CubeSoft Communications, Inc.
 * <http://www.csoft.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <libcgi/cgi.h>
#include <libcgi/mime.h>

#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>

void
mime_init_entity(struct mime_entity *ment)
{
	ment->headers = NULL;
	ment->nheaders = 0;
	ment->body = NULL;
	ment->body_len = 0;
	TAILQ_INIT(&ment->multiparts);
}

void
mime_destroy_entity(struct mime_entity *ment)
{
	int i;

	if (ment->nheaders > 0) {
		for (i = 0; i < ment->nheaders; i++) {
			free(ment->headers[i]);
		}
		free(ment->headers);
	}
	Free(ment->body);
}

void
mime_add_header(struct mime_entity *ment, const char *fmt, ...)
{
	va_list args;
	char *buf;

	va_start(args, fmt);
	if (vasprintf(&buf, fmt, args) == -1) {
		cgi_destroy(EX_TEMPFAIL, _("Out of memory"));
	}
	va_end(args);

	if (ment->headers == NULL) {
		ment->headers = Malloc(sizeof(char *));
	} else {
		ment->headers = Realloc(ment->headers,
		    (ment->nheaders+1) * sizeof(char *));
	}
	ment->headers[ment->nheaders++] = buf;
}

void
mime_add_header_str(struct mime_entity *ment, const char *s)
{
	if (ment->headers == NULL) {
		ment->headers = Malloc(sizeof(char *));
	} else {
		ment->headers = Realloc(ment->headers,
		    (ment->nheaders+1) * sizeof(char *));
	}
	ment->headers[ment->nheaders++] = Strdup(s);
}

/* Encode MIME headers to standard output. */
void
mime_write(struct cgi_query *q, struct mime_entity *ment)
{
	int i;

	for (i = 0; i < ment->nheaders; i++) {
		cgi_write(q, ment->headers[i], strlen(ment->headers[i]));
		cgi_write(q, "\n", 1);
	}
	cgi_write(q, "\015\012\n", 3);
	if (ment->body != NULL) {
		cgi_write(q, ment->body, ment->body_len);
	}
}

