/*	$Csoft: var.c,v 1.4 2004/01/03 04:10:19 vedge Exp $	*/

/*
 * Copyright (c) 2003, 2004 CubeSoft Communications, Inc.
 * <http://www.csoft.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <libcgi/cgi.h>

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#include "table.h"

/* Set a user variable. */
struct var *
var_set(const char *key, const char *fmt, ...)
{
	struct var *var;

	SLIST_FOREACH(var, &cgi->vars, vars) {
		if (strcmp(var->key, key) == 0)
			break;
	}
	if (var == NULL) {
		var = Malloc(sizeof(struct var));
		strlcpy(var->key, key, sizeof(var->key));
		SLIST_INSERT_HEAD(&cgi->vars, var, vars);
	} else {
		free(var->value);
	}
	if (fmt != NULL) {
		va_list ap;
	
		va_start(ap, fmt);
		if (vasprintf(&var->value, fmt, ap) == -1) {
			cgi_destroy(EX_TEMPFAIL, _("Out of memory"));
		}
		va_end(ap);
		var->value_len = strlen(var->value);
	} else {
		var->value = Malloc(1);
		var->value[0] = '\0';
		var->value_len = 0;
	}
	var->global = 0;
	return (var);
}

/* Set a global variable */
struct var *
var_set_global(const char *key, const char *fmt, ...)
{
	struct var *var;

	SLIST_FOREACH(var, &cgi->vars, vars) {
		if (strcmp(var->key, key) == 0)
			break;
	}
	if (var == NULL) {
		var = Malloc(sizeof(struct var));
		strlcpy(var->key, key, sizeof(var->key));
		SLIST_INSERT_HEAD(&cgi->vars, var, vars);
	} else {
		free(var->value);
	}
	if (fmt != NULL) {
		va_list ap;
	
		va_start(ap, fmt);
		if (vasprintf(&var->value, fmt, ap) == -1) {
			cgi_destroy(EX_TEMPFAIL, _("Out of memory"));
		}
		va_end(ap);
		var->value_len = strlen(var->value);
	} else {
		var->value = Malloc(1);
		var->value[0] = '\0';
		var->value_len = 0;
	}
	var->global = 1;
	return (var);
}

/* Append a string to a variable's value. */
void
var_append(struct var *var, const char *fmt, ...)
{
	char *buf;
	size_t buf_len;
	va_list ap;
	
	va_start(ap, fmt);
	if (vasprintf(&buf, fmt, ap) == -1) {
		cgi_destroy(EX_TEMPFAIL, _("Out of memory"));
	}
	buf_len = strlen(buf);
	va_end(ap);

	var->value = Realloc(var->value, var->value_len+buf_len+1);
	memcpy(&var->value[var->value_len], buf, buf_len+1);
	var->value_len += buf_len;
	free(buf);
}

/* Fetch the value of the named user variable. */
char *
var_get(const char *key)
{
	struct var *var;

	SLIST_FOREACH(var, &cgi->vars, vars) {
		if (strcmp(var->key, key) == 0)
			break;
	}
	return (var != NULL ? var->value : NULL);
}

