/*	$Csoft: varsubst.c,v 1.4 2004/01/03 04:10:19 vedge Exp $	*/

/*
 * Copyright (c) 2003, 2004 CubeSoft Communications, Inc.
 * <http://www.csoft.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <libcgi/cgi.h>
#include <libcgi/textbuf.h>

#include <ctype.h>
#include <stdlib.h>

static size_t	varsubst(struct cgi_query *, void **, size_t);

struct cgi_filter varsubst_filter = {
	CGI_INPUT_FILTER,
	"text/html",
	varsubst
};

/* Look up a registered variable by name. */
static __inline const char *
varsubst_lookup(const char *varname)
{
	struct var *var;

	/* XXX hash */
	SLIST_FOREACH(var, &cgi->vars, vars) {
		if (strcmp(var->key, varname) == 0)
			break;
	}
	if (var != NULL) {
		return (var->value != NULL ? var->value : "(null)");
	} else {
		return ("");
	}
	return ("???");
}

static __inline int
varname_ok_char(const char c)
{
	return (isalnum(c) || c == '_');
}

/* Substitute variables and unescape ampersand characters. */
static size_t
varsubst(struct cgi_query *q, void **bufp, size_t src_len)
{
	char *src = *(char **)bufp, *srcp;
	int quot = 0;
	struct textbuf te;

	textbuf_init(&te, src_len, q->cset);

	for (srcp = src; srcp < &src[src_len];) {
		char var_name[VAR_NAME_MAX];
		char *vnamep;

		/* Escape &amp; -> & when in quotes. */
		if (*srcp == '"' || *srcp == '\'') {
			quot = !quot;
		}
		if (quot && srcp < &src[src_len-5] &&
		    strncmp(srcp, "&amp;", 5) == 0) {
			textbuf_append_char(&te, '&');
			srcp += 5;
		}

		/* Parse the variable name of form `%24name' or `$name'. */
		if (srcp[0] == '%' && srcp < &src[src_len-3] &&
		    srcp[1] == '2' && srcp[2] == '4') {
			for (srcp += 3, vnamep = &var_name[0];
			     srcp < &src[src_len-1] && varname_ok_char(*srcp) &&
			      vnamep < &var_name[sizeof(var_name)-1];
			     srcp++, vnamep++) {
				*vnamep = *srcp;
			}
			*vnamep = '\0';
			textbuf_append_utf8(&te, varsubst_lookup(var_name));
		} else if (srcp[0] == '$') {
			for (srcp += 1, vnamep = &var_name[0];
			     srcp < &src[src_len-1] && varname_ok_char(*srcp) &&
			      vnamep < &var_name[sizeof(var_name)-1];
			     srcp++, vnamep++) {
				*vnamep = *srcp;
			}
			*vnamep = '\0';
			textbuf_append_utf8(&te, varsubst_lookup(var_name));
		} else {
			textbuf_append_char(&te, *(srcp++));
		}
	}

	*bufp = Realloc(*bufp, te.len);
	memcpy(*bufp, te.buf, te.len);
	textbuf_destroy(&te);
	return (te.len);
}

